package com.gamefocal.rivenworld.game.world;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.BoundingBox;
import com.gamefocal.rivenworld.DedicatedServer;
import com.gamefocal.rivenworld.entites.net.ChatColor;
import com.gamefocal.rivenworld.entites.net.HiveNetConnection;
import com.gamefocal.rivenworld.events.game.ServerReadyEvent;
import com.gamefocal.rivenworld.events.world.WorldGenerateEvent;
import com.gamefocal.rivenworld.game.GameEntity;
import com.gamefocal.rivenworld.game.NetWorldSyncPackage;
import com.gamefocal.rivenworld.game.ai.path.WorldGrid;
import com.gamefocal.rivenworld.game.collision.CollisionManager;
import com.gamefocal.rivenworld.game.entites.generics.LightEmitter;
import com.gamefocal.rivenworld.game.entites.generics.LivingEntity;
import com.gamefocal.rivenworld.game.entites.generics.TickEntity;
import com.gamefocal.rivenworld.game.generator.WorldGenerator;
import com.gamefocal.rivenworld.game.generator.basic.*;
import com.gamefocal.rivenworld.game.heightmap.RawHeightmap;
import com.gamefocal.rivenworld.game.heightmap.UnrealHeightmap;
import com.gamefocal.rivenworld.game.skills.SkillTree;
import com.gamefocal.rivenworld.game.sounds.GameSounds;
import com.gamefocal.rivenworld.game.util.Location;
import com.gamefocal.rivenworld.game.util.RandomUtil;
import com.gamefocal.rivenworld.game.util.ShapeUtil;
import com.gamefocal.rivenworld.models.GameChunkModel;
import com.gamefocal.rivenworld.models.GameEntityModel;
import com.gamefocal.rivenworld.models.GameFoliageModel;
import com.gamefocal.rivenworld.service.*;
import com.github.czyzby.noise4j.map.Grid;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.tuple.Pair;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

public class World implements Serializable {

    public static ConcurrentLinkedQueue<UUID> pendingWorldLoads = new ConcurrentLinkedQueue<>();

    public static Float cellSize = 100f;

    public ConcurrentLinkedQueue<UUID> tickEntites = new ConcurrentLinkedQueue<>();

    public ConcurrentHashMap<UUID, WorldChunk> entityChunkIndex = new ConcurrentHashMap<>();

    public WorldGenerator generator;
    public ConcurrentHashMap<Location, String> chunkVersions = new ConcurrentHashMap<>();
    public ConcurrentLinkedQueue<WorldChunk> dirtyChunks = new ConcurrentLinkedQueue<>();
    private Hashtable<String, Grid> layers = new Hashtable<>();
    private WorldChunk[][] chunks = new WorldChunk[0][0];
    private Pair<Integer, Integer> chunkPointer = Pair.of(0, 0);

    private CollisionManager collisionManager;

    private int chunkSize = 24;

    private UnrealHeightmap heightmap;

    private RawHeightmap rawHeightmap;

    private WorldGrid grid;

    public World() {
        /*
         * Load the world into Memory
         * */
//        try {
//            List<GameEntityModel> entites = DataService.gameEntities.queryForAll();
//
//            for (GameEntityModel model : entites) {
//                this.entites.put(model.uuid, model);
//            }
//
//        } catch (SQLException throwables) {
//            throwables.printStackTrace();
//        }

        try {
            FileUtils.forceMkdir(new File("data"));
            ClassLoader classloader = Thread.currentThread().getContextClassLoader();

            try (InputStream in = classloader.getResourceAsStream("rivenworld_full.png")) {

                // Default
//                byte[] b = IOUtils.toByteArray(reader);

//                Files.write(Paths.get("data/map.png"), b, StandardOpenOption.CREATE);

                Files.copy(in, Path.of("data/map.png"), StandardCopyOption.REPLACE_EXISTING);

            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        System.out.println("Generating Heightmap...");
//        Heightmap heightmap = new Heightmap();
//        heightmap.loadFromImageSet("data/map.png");

//        this.heightmap = new UnrealHeightmap(
//                new String[]{
//                        "tiles/rw_x0_y0.png",
//                        "tiles/rw_x0_y1.png",
//                        "tiles/rw_x0_y2.png",
//                        "tiles/rw_x0_y3.png",
//                        "tiles/rw_x1_y0.png",
//                        "tiles/rw_x1_y1.png",
//                        "tiles/rw_x1_y2.png",
//                        "tiles/rw_x1_y3.png",
//                        "tiles/rw_x2_y0.png",
//                        "tiles/rw_x2_y1.png",
//                        "tiles/rw_x2_y2.png",
//                        "tiles/rw_x2_y3.png",
//                        "tiles/rw_x3_y0.png",
//                        "tiles/rw_x3_y1.png",
//                        "tiles/rw_x3_y2.png",
//                        "tiles/rw_x3_y3.png",
//                }, 252, 252, 17
//        );

        this.rawHeightmap = new RawHeightmap((int) Math.ceil(151393f / 100f), "world.bin");

        System.out.println("Creating World Generator...");
        this.generator = new WorldGenerator(this.heightmap,
                new SmallRockLayer(),
                new StickLayer(),
                new FiberLayer(),
                new FoodLayer(),
                new GoldLayer(),
                new CoalLayer(),
                new IronLayer(),
                new MineralLayer(),
                new BirdNestLayer()
        );

        this.chunks = this.getWorldCells(this.chunkSize * 100);
        this.collisionManager = new CollisionManager(201600);

        // Load the grid
        this.grid = new WorldGrid(this);
    }

    public static void generateNewWorld() {
        System.out.println("[WORLD]: Generating new World...");
        // Generate a new world...
        World world = DedicatedServer.instance.getWorld();

        System.out.println("[WORLD]: Loading Chunks...");
        for (int x = 0; x < world.getChunks().length; x++) {
            for (int y = 0; y < world.getChunks().length; y++) {

                WorldChunk c = world.getChunk(x, y);

                GameChunkModel chunkModel = new GameChunkModel();
                chunkModel.id = c.getChunkCords();
                chunkModel.claim = null;
                chunkModel.conflictStart = 0L;
                chunkModel.isPrimaryChunk = false;
                chunkModel.conflictTimer = 0L;
                chunkModel.inConflict = false;

                DataService.exec(() -> {
                    try {
                        DataService.chunks.createOrUpdate(chunkModel);

                        System.out.println("Chunk " + c.getX() + "," + c.getY() + " Saved.");
                    } catch (SQLException throwables) {
                        throwables.printStackTrace();
                    }
                });
            }
        }

        DataService.exec(() -> {
            System.out.println("[WORLD]: Generating Resource Layers...");
            world.generator.run(DedicatedServer.instance.getWorld());
        });

        DataService.exec(() -> {
            System.out.println("[WORLD]: GENERATION COMPLETE.");
            DedicatedServer.isReady = true;
            new WorldGenerateEvent().call();
            System.out.println("Server Ready.");
            new ServerReadyEvent().call();
        });
    }

    public void worldToFile() throws IOException {
        FileOutputStream f = new FileOutputStream(new File("_world.bin"));
        ObjectOutputStream o = new ObjectOutputStream(f);
        o.writeObject(this);
        o.close();
        f.close();
    }

    public RawHeightmap getRawHeightmap() {
        return rawHeightmap;
    }

    public WorldGrid getGrid() {
        return grid;
    }

    public UnrealHeightmap getHeightmap() {
        return heightmap;
    }

    public CollisionManager getCollisionManager() {
        return collisionManager;
    }

    public void prepareWorld() {
//        try {
//            List<GameEntityModel> models = DataService.gameEntities.queryForAll();
//            for (GameEntityModel m : models) {
//                this.entityTree.push(m);
//            }
//        } catch (SQLException throwables) {
//            throwables.printStackTrace();
//        }

        /*
         * Migrate anything to new blocks.
         * */
        BackwardsCompatibilityService.migrate();

        DataService.exec(() -> {
            for (int x = 0; x < this.chunks.length; x++) {
                for (int y = 0; y < this.chunks.length; y++) {
                    WorldChunk c = this.getChunk(x, y);
                    c.loadEntitesIntoMemory();
                    this.chunkVersions.put(c.getChunkCords(), c.chunkHash());
                }
            }

            DedicatedServer.isReady = true;
            System.out.println("Server Ready.");
            new ServerReadyEvent().call();
        });
    }

    public Location getNearbyLocationWithNoCollision(Location location, float searchRadius) {
        Random random = new Random();
        Vector3 pos = new Vector3(location.getX(), location.getY(), location.getZ());
        BoundingBox searchArea = new BoundingBox(pos.cpy().sub(searchRadius, searchRadius, searchRadius), pos.cpy().add(searchRadius, searchRadius, searchRadius));

        for (int i = 0; i < 1000; i++) { // max attempts to find an open area
            Vector3 spawnLocation = searchArea.min.cpy().lerp(searchArea.max, random.nextFloat());
            BoundingBox bagBox = new BoundingBox(spawnLocation, spawnLocation.cpy().add(1, 1, 1)); // Assuming a Bag is 1x1x1 units
            boolean collides = false;

            List<GameEntity> nearbyEntities = collisionManager.getNearbyEntities(new Location(spawnLocation.x, spawnLocation.y, spawnLocation.z));
            for (GameEntity entity : nearbyEntities) {
                if (collisionManager.isColliding(entity, bagBox)) {
                    collides = true;
                    break;
                }
            }

            if (!collides) {
                return new Location(spawnLocation.x, spawnLocation.y, spawnLocation.z);
            }
        }

        return null;
    }

    public Grid getLayer(String name) {
        if (this.layers.containsKey(name)) {
            return this.layers.get(name);
        }

        return null;
    }

    public int getChunkSize() {
        return chunkSize;
    }

    public void setLayer(String name, Grid grid) {
        this.layers.put(name, grid);
    }

    public Location randomLocationInWorld() {
        float x = RandomUtil.getRandomNumberBetween(0, 1008);
        float y = RandomUtil.getRandomNumberBetween(0, 100);
        return this.generator.getHeightmap().getWorldLocationFrom2DMap(new Location(x, y, 0));
    }

    public Location randomLocationInGrid(Grid grid, float minVal, float minHeight) {
        ArrayList<Location> locations = new ArrayList<>();

        for (int x = 0; x < grid.getWidth(); x++) {
            for (int y = 0; y < grid.getHeight(); y++) {
                float v = grid.get(x, y);
                float h = this.generator.getHeightmap().getHeightFrom2DLocation(new Location(x, y, 0));
                if (v >= minVal && h >= minHeight) {
                    locations.add(this.generator.getHeightmap().getWorldLocationFrom2DMap(new Location(x, y, 0)));
                }
            }
        }

        return RandomUtil.getRandomElementFromList(locations);
    }

    public List<GameEntity> findCollisionEntites(Location location, float radius) {
        ArrayList<GameEntity> hitList = new ArrayList<>();
        WorldChunk chunk = this.getChunk(location);
        if (chunk != null) {
            // Has a chunk
            hitList.addAll(chunk.getCollisionEntites());
            for (WorldChunk c : chunk.neighbors()) {
                hitList.addAll(c.getCollisionEntites());
            }
        }
        return hitList;
    }

    public void loadWorldForPlayer(HiveNetConnection connection) {
        connection.disableWorldSync();
        DedicatedServer.get(InventoryService.class).trackInventory(connection.getPlayer().inventory);

        new Thread(() -> {

            connection.displayLoadingScreen("Initializing...", 0.0f);
            connection.hide();
            connection.playBackgroundSound(GameSounds.BG2, 1f, 1f);
            connection.sendSyncPackage(true);
            connection.sendStatePacket(connection);

            /*
             * Sync foliage that is cut or destroyed
             * */
//            try {
            Collection<GameFoliageModel> foliageModels = DedicatedServer.get(FoliageService.class).getFoliage().values();

            connection.displayLoadingScreen("Loading Foliage", 0.0f);

            int i = 0;
            for (GameFoliageModel foliageModel : foliageModels) {
                String currentHash = foliageModel.stateHash();
                String syncHash = "NONE";

                connection.displayLoadingScreen("Loading Foliage (" + i++ + "/" + foliageModels.size() + ")", (float) i / (float) foliageModels.size());

                foliageModel.syncToPlayer(connection, false);

                try {
                    Thread.sleep(2);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
//            } catch (SQLException throwables) {
//                throwables.printStackTrace();
//            }


            connection.displayLoadingScreen("Loading World", 0.0f);

            int totalChunks = DedicatedServer.instance.getWorld().getChunks().length * DedicatedServer.instance.getWorld().getChunks()[0].length;

            i = 0;
            for (WorldChunk[] chunks : DedicatedServer.instance.getWorld().getChunks()) {
                for (WorldChunk chunk : chunks) {
                    // Loop through each chunk

                    connection.displayLoadingScreen("Loading Chunk " + chunk.getChunkCords().getX() + "," + chunk.getChunkCords().getY(), (float) i++ / (float) totalChunks);

                    connection.subscribeToChunk(chunk);

                    NetWorldSyncPackage syncPackage = new NetWorldSyncPackage();
                    connection.syncChunkLOD(chunk, true, true, syncPackage);
                    connection.sendWorldStateSyncPackage(syncPackage);

//                    connection.syncChunkLOD(chunk, true, true, true);

                    try {
                        Thread.sleep(1);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }

//                    if (inView && !isLoaded) {
//                        // Is in view but not loaded
//                        connection.subscribeToChunk(chunk);
//                    } else if (isLoaded && !inView) {
//                        // Is loaded but no longer in view
//                        connection.unsubscribeToChunk(chunk);
//                    } else if (inView && isLoaded) {
//                        // Is loaded and in view, update entites
//                        for (GameEntityModel entityModel : chunk.getEntites().values()) {
//                            connection.syncEntity(entityModel, chunk, false, true);
//                        }
//                    }
                }
            }

            DedicatedServer.get(EnvironmentService.class).emitEnvironmentChange(connection);
            connection.displayLoadingScreen("Loading Environment", 0.1f);

            connection.setSkillTree(new SkillTree(connection));
            connection.displayLoadingScreen("Loading Skills", 0.15f);

            connection.displayLoadingScreen("Syncing Inventory", 0.2f);
            connection.updatePlayerInventory();

//            connection.displayLoadingScreen("Syncing Equipment", 0.3f);
//            connection.syncEquipmentSlots();

            connection.displayLoadingScreen("Loading Other Players", 0.3f);
//            connection.syncEquipmentSlots();

//            for (HiveNetConnection c : DedicatedServer.get(PlayerService.class).players.values()) {
//                // Send move event to them for everyone else
//                if (connection.getUuid() != c.getUuid()) {
//                    HiveNetMessage message = new HiveNetMessage();
//                    message.cmd = "plmv";
//                    message.args = new String[]{c.getUuid().toString(), String.valueOf(c.getVoiceId()), c.getPlayer().location.toString()};
//                    connection.sendUdp(message.toString());
//                }
//            }

            connection.displayLoadingScreen("Syncing Equipment", 0.40f);
            connection.syncEquipmentSlots();

            connection.enableWorldSync();
            connection.displayLoadingScreen("Preparing Spawn", 0.90f);
            connection.syncToAmbientWorldSound();
            connection.tpToLocation(connection.getPlayer().location);
            connection.show();
            connection.enableMovment();
            connection.setLoaded(true);
//            connection.sendTcp("FP|t");
//            connection.setFP(true);
//            connection.sendTcp("loadmap|");
            connection.hideLoadingScreen();

            /*
             * Send a join msg
             * */
            if (!connection.isAdmin()) {
                DedicatedServer.sendChatMessageToAll(ChatColor.GREEN + "" + connection.getPlayer().displayName + " has joined the game");
            }
        }).start();
    }

    public GameEntityModel spawn(GameEntity entity, Location location) {
        return this.spawn(entity, location, null);
    }

    public GameEntityModel spawn(GameEntity entity, Location location, HiveNetConnection owner) {

        if (entity.uuid == null) {
            entity.uuid = UUID.randomUUID();
        }

        if (entity.location == null) {
            entity.location = location;
        }

        WorldChunk spawnChunk = this.getChunk(location);
        if (spawnChunk != null) {
            GameEntityModel model = spawnChunk.spawnEntity(entity, location, owner, false);
            DedicatedServer.instance.getWorld().entityChunkIndex.put(model.uuid, spawnChunk);

            model.entityData.calcChunkHash();

            if (TickEntity.class.isAssignableFrom(model.entityData.getClass())) {
                // Is a Tick Entity
                this.tickEntites.add(model.uuid);
            }

            if (LivingEntity.class.isAssignableFrom(model.entityData.getClass())) {
                DedicatedServer.get(AiService.class).trackedEntites.add(entity.uuid);
            }

            if (LightEmitter.class.isAssignableFrom(model.entityData.getClass())) {
                DedicatedServer.get(AiService.class).lightSources.put(model.entityData.uuid, model.entityData);
            }

            // Add to collision manager
            this.collisionManager.addEntity(entity);

            return model;
        } else {
//            System.err.println("Invalid Chunk...");
        }

        return null;
    }

    public void playSoundAtLocation(GameSounds sound, Location at, float radius, float volume, float pitch, float timeInSeconds) {
        for (HiveNetConnection connection : DedicatedServer.get(PlayerService.class).players.values()) {
            if (at.dist(connection.getPlayer().location) <= (radius * 100)) {
                connection.playLocalSoundAtLocation(sound, at, volume, pitch, timeInSeconds);
            }
        }
    }

    public void playSoundAtLocation(GameSounds sound, Location at, float radius, float volume, float pitch) {
        for (HiveNetConnection connection : DedicatedServer.get(PlayerService.class).players.values()) {
            if (at.dist(connection.getPlayer().location) <= (radius * 100)) {
                connection.playLocalSoundAtLocation(sound, at, volume, pitch, -1);
            }
        }
    }


    public void playSoundToAllPlayers(GameSounds sound, float volume, float pitch) {
        for (HiveNetConnection connection : DedicatedServer.get(PlayerService.class).players.values()) {
            connection.playSoundAtPlayer(sound, volume, pitch);
        }
    }

    public void playSoundToAllPlayersWithinRadius(GameSounds sound, Location source, float radius, float volume, float pitch) {
        for (HiveNetConnection connection : DedicatedServer.get(PlayerService.class).players.values()) {
            if (source.dist(connection.getPlayer().location) <= (radius * 100)) {
                connection.playSoundAtPlayer(sound, volume, pitch);
            }
        }
    }

    public void despawn(UUID uuid) {
        if (this.entityChunkIndex.containsKey(uuid)) {
            GameEntityModel m = this.entityChunkIndex.get(uuid).getEntites().get(uuid);
            this.collisionManager.removeEntity(m.entityData);
            this.getGrid().refreshOverlaps(m.entityData.getBoundingBox());
            SaveService.removeSave(m);
            this.entityChunkIndex.get(uuid).despawnEntity(uuid);
            this.tickEntites.remove(uuid);
            DedicatedServer.get(AiService.class).trackedEntites.remove(uuid);
            DedicatedServer.get(AiService.class).lightSources.remove(uuid);
        }
//        if (this.entites.containsKey(uuid)) {
//            EntityDespawnEvent event = new EntityDespawnEvent(this.entites.get(uuid));
//            if (event.isCanceled()) {
//                return;
//            }
//
//            GameEntityModel model = event.getModel();
//
//            DataService.exec(() -> {
//                try {
//                    model.despawn();
//                    DedicatedServer.instance.getWorld().entites.remove(model.uuid);
//                    DataService.gameEntities.delete(model);
//                } catch (SQLException throwables) {
//                    throwables.printStackTrace();
//                }
//            });
//        }
    }

    public boolean hasEntityOfUUID(UUID uuid) {
        if (this.entityChunkIndex.containsKey(uuid)) {
            return this.entityChunkIndex.get(uuid).hasEntity(uuid);
        }

        return false;
    }

    public void save() {
        for (HiveNetConnection connection : DedicatedServer.get(PlayerService.class).players.values()) {
            DataService.exec(() -> {
                try {
                    DataService.players.update(connection.getPlayer());
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            });
        }
        DataService.exec(() -> {
            System.out.println("World Entity Save Complete.");
        });
    }

    public boolean isFreshWorld() {
        try {
            return (DataService.gameEntities.countOf() == 0);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }

        return true;
    }

    public <T> List<T> getEntitesOfTypeWithinRadius(Class<T> t, Location base, float radius) {
        radius *= cellSize;

        ArrayList<T> matches = new ArrayList<>();
//        for (GameEntityModel m : this.entites.values()) {
//            if (m.entityData.getClass().isAssignableFrom(t)) {
//                if (m.location.dist(base) <= radius) {
//                    matches.add(m.getEntity(t));
//                }
//            }
//        }
//        for (WorldChunk c : this.getChunksAroundLocation(base, radius)) {
        for (GameEntity m : this.collisionManager.getNearbyEntities(base)) {
            if (m.getClass().isAssignableFrom(t)) {
                if (m.location.dist(base) <= radius) {
                    matches.add((T) m);
                }
            }
        }
//        }

        return matches;
    }

    public <T> T getClosestEntityOfTypeWithinRadius(Class<T> t, Location base, float radius) {
        ArrayList<T> l = (ArrayList<T>) this.getEntitesOfTypeWithinRadius(t, base, radius);
        if (l.size() > 0) {

            l.sort((o1, o2) -> {
                GameEntity _1 = (GameEntity) o1;
                GameEntity _2 = (GameEntity) o2;

                float dst1 = base.dist(_1.location);
                float dst2 = base.dist(_2.location);

                if (dst1 < dst2) {
                    return -1;
                } else if (dst1 > dst2) {
                    return +1;
                }

                return 0;
            });

            return l.get(0);
        }

        return null;
    }

    public <T> List<T> getEntitesOfType(Class<T> type) {
        ArrayList<T> l = new ArrayList<>();
//        for (GameEntityModel m : this.entites.values()) {
//            if (type.isAssignableFrom(m.entityData.getClass())) {
//                l.add((T) m.entityData);
//            }
//        }

        for (UUID u : this.entityChunkIndex.keySet()) {
            GameEntityModel m = this.getEntityFromId(u);
            if (m != null) {
                if (type.isAssignableFrom(m.entityData.getClass())) {
                    l.add((T) m.entityData);
                }
            }
        }

        return l;
    }

    public GameEntityModel getEntityFromId(UUID uuid) {
        if (this.entityChunkIndex.containsKey(uuid)) {

            return this.entityChunkIndex.get(uuid).getEntityModelFromUUID(uuid);
        }
//        if (this.entites.containsKey(uuid)) {
//            return this.entites.get(uuid);
//        }
        return null;
    }

    public void entityChunkUpdate(GameEntityModel model) {
        if (model != null && model.entityData != null && model.entityData.hasMovedChunks()) {
            // Moved
            WorldChunk indexChunk = this.entityChunkIndex.get(model.uuid);
            indexChunk.getEntites().remove(model.uuid);

            indexChunk.version = System.currentTimeMillis();

            WorldChunk newChunk = model.entityData.getChunk();
            newChunk.getEntites().put(model.uuid, model);

            newChunk.version = System.currentTimeMillis();

            indexChunk.markDirty();
            newChunk.markDirty();

            this.entityChunkIndex.put(model.uuid, newChunk);
            model.entityData.calcChunkHash();
        }
    }

    public void updateEntity(GameEntityModel model) {
        if (this.entityChunkIndex.containsKey(model.uuid)) {
            model.version = System.currentTimeMillis();

            this.entityChunkIndex.get(model.uuid).updateEntity(model);

            model.entityData.forceUpdate();

//            this.entites.put(model.uuid, model);

//            DataService.exec(() -> {
//                try {
//                    DataService.gameEntities.update(model);
//                } catch (SQLException throwables) {
//                    throwables.printStackTrace();
//                }
//            });

        } else {
//            System.err.println("Failed to update entity that does not exist");
        }
    }

    public WorldChunk getChunk(Location location) {

//        Location loc2d = this.generator.getHeightmap().getMappedLocationFromGame(location);

        Location zeroBasedLoc = this.toZeroBasedCords(location);

        float x = (zeroBasedLoc.getX() / (this.chunkSize * 100));
        float y = (zeroBasedLoc.getY() / (this.chunkSize * 100));

        return this.getChunk(x, y);
    }

    public Location getChunkCordsFromLocation(Location location) {
        Location zeroBasedLoc = this.toZeroBasedCords(location);
        float x = (zeroBasedLoc.getX() / (this.chunkSize * 100));
        float y = (zeroBasedLoc.getY() / (this.chunkSize * 100));
        return new Location(x, y, 0);
    }

    public WorldChunk getChunk(int x, int y) {
        try {
            return this.chunks[x][y];
        } catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }

    public WorldChunk getChunk(float x, float y) {
        return this.getChunk((int) x, (int) y);
    }

    public WorldChunk[][] getWorldCells(int size) {

        WorldChunk[][] chunks = new WorldChunk[(201600 / size)][(201600 / size)];


        for (int x = 0; x < chunks.length; x++) {
            for (int y = 0; y < chunks.length; y++) {
                // Each 1 unit is 100 in-game. Each chunk is 20 units.
                chunks[x][y] = new WorldChunk(this, x, y);
            }
        }

        return chunks;
    }

    public WorldChunk[][] getChunks() {
        return chunks;
    }

    public ArrayList<WorldChunk> getChunksInBox(BoundingBox boundingBox) {
        ArrayList<WorldChunk> chunks = new ArrayList<>();

        Rectangle rectangle = ShapeUtil.boundingBoxToRectangle(boundingBox);

        for (WorldChunk[] cc : this.chunks) {
            for (WorldChunk c : cc) {
                if (rectangle.overlaps(c.box) || rectangle.contains(c.box)) {
                    chunks.add(c);
                }
            }
        }

        return chunks;
    }

    public Location toZeroBasedCords(Location location) {
        return new Location(
                MathUtils.map(-25181.08f, 176573.27f, 0, 201754, location.getX()),
                MathUtils.map(-25181.08f, 176573.27f, 0, 201754, location.getY()),
                location.getZ()
        );
    }

    public Location fromZeroBasedCords(Location location) {
        return new Location(
                MathUtils.map(0, 201754, -25181.08f, 176573.27f, location.getX()),
                MathUtils.map(0, 201754, -25181.08f, 176573.27f, location.getY()),
                location.getZ()
        );
    }

    public boolean isChunkInView(Location location, float radius, WorldChunk chunk) {
        BoundingBox searchBox = ShapeUtil.makeBoundBox(location.cpy().setZ(0).toVector(), radius, 60000);
        BoundingBox boundingBox = chunk.getBoundingBox();
        return boundingBox.intersects(searchBox) || boundingBox.contains(searchBox);
    }

    public List<WorldChunk> getChunksAroundLocation(Location location, float radius) {
        ArrayList<WorldChunk> chunks = new ArrayList<>();
        BoundingBox searchBox = ShapeUtil.makeBoundBox(location.cpy().setZ(0).toVector(), radius, 60000);

        for (int x = 0; x < this.chunks.length; x++) {
            for (int y = 0; y < this.chunks.length; y++) {
                WorldChunk chunk = this.chunks[x][y];
                BoundingBox boundingBox = chunk.getBoundingBox();
                if (boundingBox.intersects(searchBox) || boundingBox.contains(searchBox)) {
                    chunks.add(chunk);
                }
            }
        }

//        for (WorldChunk chunk : this) {
//            BoundingBox boundingBox = chunk.getBoundingBox();
//            if (boundingBox.intersects(searchBox) || boundingBox.contains(searchBox)) {
//                chunks.add(chunk);
//            }
//        }
        return chunks;
    }

    public void updateEntity(GameEntity model) {
        if (this.entityChunkIndex.containsKey(model.uuid)) {
            GameEntityModel m = this.entityChunkIndex.get(model.uuid).getEntityModelFromUUID(model.uuid);
            this.updateEntity(m);
//            m.entityData = model;
//            this.entites.put(model.uuid, m);
        } else {
            System.err.println("Failed to update entity that does not exist");
        }
    }
}
